/*  atenea
 *  Copyright (C) 1999 <ryu@mundivia.es>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#define USE_PLUGIN

#include "../xchat.h"
#include "../plugin.h"
#include "rules.h"
#include <glib.h>

extern	struct module *module_find (char *name);

int atenea_cmd (struct session *sess, char *tbuf, char *word[], char *word_eol[]);
int atenea_privmsg (struct server *serv, char *from, char *ip, char *text, void *a, char c);
int atenea_handler(); /* For the timer */
void pending_add(struct session *sess,char *cmd, int timeout);

char	*name = "Atenea";
char	*desc = "Rule based auto query replier.";

struct	commands	atenea_cmds[] = {
	{"ATENEA",	atenea_cmd,	0, 0, "/ATENEA <command> [<parameters>,...]"},
	{0, 0, 0, 0, 0}
};

struct	xp_signal	privmsg_sig;
int	(*privmsg_next)	(void *, void *, void *, void *, void *, char);

struct	module_cmd_set	atenea_cmd_set;

GList *pending_lines=NULL;

typedef struct{
  struct session *sess;
  char *comando;
  int timeout;
}Pending;

GList *rules;
GList *default_say;
extern FILE *yyin;
char conffile[512]; // It's global for usage in RESET command

extern int yyparse(void);
extern void PrintText(struct session *,char *);
extern void hook_signal(struct xp_signal *);
extern void module_add_cmds(struct module_cmd_set *);
extern void handle_command(char *, struct session *, int);
extern void module_cleanup (struct module *mod, struct session *sess);

int atenea_timeout;

int load_rules(char *file)
{
  int res;
  yyin=fopen(file,"ro");
  if (yyin!=NULL)
     {
     res=yyparse();
     fclose(yyin);
     return (!res);
     }  
return 0;
}

int	module_init (int ver, struct module *mod, struct session *sess)
{
	/* Comprueba que el xchat es de una version adecuada */
	if (ver != MODULE_IFACE_VER)
		return 1;
	
	if (module_find (name) != NULL) {
		/* Ya est cargado de antes */
		PrintText(sess, "Atenea was already loaded.\n");
		return 1;
	}
	PrintText(sess, "Loading Atenea...\n");
	mod->name = name;
	mod->desc = desc;
		
	privmsg_sig.signal = XP_PRIVMSG;
	privmsg_sig.callback = XP_CALLBACK(atenea_privmsg);
	privmsg_sig.naddr = &privmsg_next;
	privmsg_sig.mod = mod;
	
	hook_signal(&privmsg_sig);

	atenea_cmd_set.mod = mod;
	atenea_cmd_set.cmds = atenea_cmds;
	
	module_add_cmds (&atenea_cmd_set);
	
        srand((unsigned int)malloc(1));
	atenea_timeout=gtk_timeout_add(1000,(GtkFunction)atenea_handler,0);
      
        strncpy(conffile,g_get_home_dir(),512);
        strncat(conffile,"/.atenea.brain",512);
        if (load_rules(conffile)) 
	     {
	     PrintText(sess,"\0034,1                                    \n\000");
	     PrintText(sess,"\0034,1 \0034,14 \0034,5 \0034,7 \0034,8                            \0034,7 \0034,5 \0034,14 \0034,1 \n\000");
	     PrintText(sess,"\0034,1 \0034,14 \0034,5 \0034,7 \0034,8 Atenea successfully loaded \0034,7 \0034,5 \0034,14 \0034,1 \n\000");
	     PrintText(sess,"\0034,1 \0034,14 \0034,5 \0034,7 \0034,8                            \0034,7 \0034,5 \0034,14 \0034,1 \n\000");
	     PrintText(sess,"\0034,1                                    \n\000");
	     return 0;
	     }
	PrintText(sess,"Error loading main brain (~/.atenea.brain)\n");
	module_cleanup(mod,sess);
	return 1;
}

void	module_cleanup (struct module *mod, struct session *sess)
{
	PrintText(sess, "Unloading Atenea.\n");
	gtk_timeout_remove(atenea_timeout);
	g_list_free(default_say);
	g_list_free(pending_lines);
}

int stroccur(char *orig, char *delim)
{
  char *p=orig;
  int count=0;
  while((p=strstr(p,delim))!=NULL)
    {
      count++;
      p++;
    }
  return count;
}

void normalize(char *s)
{
  int i;
  for (i=0;i<strlen(s);i++)
    {
      if (s[i]>='A' && s[i]<='Z')
        {
          s[i]+='a'-'A';
        }
      else switch (s[i]) {
      	     case '': s[i]='a';break;
      	     case '': s[i]='e';break;
      	     case '': s[i]='i';break;
      	     case '': s[i]='o';break;
      	     case '': s[i]='u';break;
      	     case '': s[i]='a';break;
      	     case '': s[i]='e';break;
      	     case '': s[i]='i';break;
      	     case '': s[i]='o';break;
      	     case '': s[i]='u';break;
      	     case '': s[i]='a';break;
      	     case '': s[i]='e';break;
      	     case '': s[i]='i';break;
      	     case '': s[i]='o';break;
      	     case '': s[i]='u';break;
      	     case '': s[i]='a';break;
      	     case '': s[i]='e';break;
      	     case '': s[i]='i';break;
      	     case '': s[i]='o';break;
      	     case '': s[i]='u';break;
      	     case '': s[i]='a';break;
      	     case '': s[i]='e';break;
      	     case '': s[i]='i';break;
      	     case '': s[i]='o';break;
      	     case '': s[i]='u';break;
      	     case '': s[i]='a';break;
      	     case '': s[i]='e';break;
      	     case '': s[i]='i';break;
      	     case '': s[i]='o';break;
      	     case '': s[i]='u';break;
      	     case '': s[i]='a';break;
      	     case '': s[i]='e';break;
      	     case '': s[i]='i';break;
      	     case '': s[i]='o';break;
      	     case '': s[i]='u';break;
      	     case '': s[i]='a';break;
      	     case '': s[i]='e';break;
      	     case '': s[i]='i';break;
      	     case '': s[i]='o';break;
      	     case '': s[i]='u';break;
      	     }
    }
}

char *strchg(char *orig, char *source, char *target)
{
  char *result;
  int count=stroccur(orig,source);
  char *nuevo=strdup(orig);
  char *search=strstr(nuevo,source);
  char *last=nuevo;
  result=(char *)malloc(strlen(orig)+count*(strlen(target)-strlen(source))+2);
  strcpy(result,"");
  while (search!=NULL)
    {
      *search=0;
      strcat(result,last);
      strcat(result,target);
      search+=strlen(source);
      last=search;
      search=strstr(search,source);
    }
  strcat(result,last);
  free(nuevo);
  return result;
}

int	atenea_privmsg (struct server *serv, char *from, char *ip, char *text, void *a, char c)
{
	struct	session	*sess;
	char	*changed,buf[512];
	GList   *to_say, *index;
	int     delay;
	
	sess = serv->front_session;
	normalize(text);
	to_say=atenea_say(text,rules,default_say);
	to_say=g_list_reverse(to_say);
	index=to_say;
	while(index!=NULL)
	  {
	    changed=strchg((char *)index->data,"$nick",from);
            delay=(strlen(changed)/4)+1;
            if (changed[0]!='/')
              {
	      snprintf(buf, 510, "/msg %s %s\n", from, changed);
	      pending_add(sess,strdup(buf),delay);
              }
            else pending_add(sess,strdup(changed),delay);
	    index=index->next;
            free(changed);
          }
        g_list_free(to_say);
	XP_CALLNEXT(privmsg_next, serv, from, ip, text, a, c);
}

void print_help(struct session *sess)
{
  PrintText(sess,"Atenea usage:\n");
  PrintText(sess,"/ATENEA <command> [...]\n");
  PrintText(sess,"Commands:\n");
  PrintText(sess,"	* RESET:       Reloads default rules file.\n");
  PrintText(sess,"	* STOP:        Stops auto-reply.\n");
  PrintText(sess,"	* CONTINUE:    Continues auto-reply.\n");
  PrintText(sess,"	* LOAD <file>: Loads an alternative brain.\n");
}

int	atenea_cmd (struct session *sess, char *tbuf, char *word[], char *word_eol[])
{
	PrintText(sess,"\0034,1 \0034,14 \0034,5 \0034,7 \0034,8 Atenea Control \0034,7 \0034,5 \0034,14 \0034,1 \n\000");
	if (strlen(word[2])==0)
	  {
	    print_help(sess);
	  }
	else
	  {
	    if (!strcasecmp(word[2],"reset"))
	       {
	         if (!load_rules(conffile)) PrintText(sess,"Error loading rules (see standard error)\n");
	         else PrintText(sess,"New rules successfully loaded\n");
	       }
	    else if (!strcasecmp(word[2],"stop"))
	       {
	         PrintText(sess,"Not implemented yet\n");
	       }
	    else if (!strcasecmp(word[2],"load"))
	       {
	         if (!load_rules(word[3])) PrintText(sess,"Error loading rules (see standard error)\n");
	         else PrintText(sess,"New rules successfully loaded\n");
	       }
	    else print_help(sess);
	  }
	return 0;
}

int atenea_handler()
{
  Pending *p;
  if (pending_lines==NULL) return 1;
  
  p=(Pending *)pending_lines->data;

  p->timeout--;
  if (p->timeout==0)
    {
      handle_command(p->comando,p->sess,0);
      free(p->comando);
      pending_lines=g_list_remove(pending_lines,p);
      free(p);
    }
return 1;
}

void pending_add(struct session *sess,char *cmd, int timeout)
{
  Pending *p=(Pending *)malloc(sizeof(Pending));
  p->sess=sess;
  p->comando=cmd;
  p->timeout=timeout;
  pending_lines=g_list_append(pending_lines,p);
}
